/******************************************************************************
Filename:     modPwmImpl.c
Target: CC2530 of TI

Description:
H.Zheng MCU building block layer module library: Timer part

ӦÿͨӿʵPWM

ʵʹTimer1 channel2P1_0
ÿtickΪ1/32(us)PWMɳʼеvPeriodTicks
ΪvPeriodTicks/32(us)

modification:
  2014.03.23 creation Zheng Hui
  2014.08.23 modified to control LED P1_0, use Timer1 channel 2, alt config.


COPYRIGHT
Zheng Hui, Guangzhou, China. hzheng@gzhu.edu.cn. Guangzhou University

******************************************************************************/
#include <ioCC2530.h>
#include "modPwm.h"


static modPwmCallbackFunction* mPwmCallback=0;
static uint8 mPwmNumOfOverflowCfg;
static uint8 mPwmNumOfOverflowLeft;


/**
 * PWMܳʼӿ
 * 
 * vChannel0ʼײʵֶ֧·PWMźʱñָʾ·źŽá
 * vPeriodTicksźNλΪtick1 tick೤ʱɵײʵĵָ
 *               N=0ʱʾֹͣchannelPWMܡ
 * vOverflowCountsTimerﵽvOverflowCountsʱصá
 * vFunctionûĻص
 * ʵ֣ʹTimer1 Alt.2, channel2, P1_0
 */
int8 modPwmInit(uint16 vPeriodTicks, uint8 vOverflowCounts, modPwmCallbackFunction* vFunction){
  //CC2530: ʹTimer1ΪPWMʱ

  //disable timer 1
  T1CTL=0x00;
 
  //save input parameters
  mPwmCallback = vFunction;
  mPwmNumOfOverflowCfg = vOverflowCounts;
  mPwmNumOfOverflowLeft = vOverflowCounts;
  
  //disable all 4 timer1 channels
  T1CCTL0=0x00;
  T1CCTL1=0x00;
  T1CCTL2=0x00;
  T1CCTL3=0x00;
  T1CCTL4=0x00;

  //clear Timer1 overflow interrupt mask, TIMIF.6
  TIMIF=TIMIF&0x0BF; //10111111

  //clear overflow & channel interrupt flag
  T1STAT = 0x00;

  //check whether stop PWM function
  if (vPeriodTicks==0) {
	return 0;
  }
  
  
  //I/O 
  //PERCFG.T1CFG=0: Timer1ʹAlt.2 I/Oӳ䷽ʽchannel 2 ӦP1_0
  PERCFG|=0x40;
  //P1_0Ϊ
  P1SEL|=0x01;
  P1DIR|=0x01;
  //Timer1ŵʹȨ
  P2SEL=(P2SEL&0x0EF)|0x08; //clear P2SEL.PRI1P1[4], set P2SEL.PRI0P1[3]
  P2DIR=(P2DIR|0x80)&(0x0BF); //set P2DIR.PRIP0[76]=0x10

  //set PWM
  
  T1CC0L=vPeriodTicks&0x0ff;
  T1CC0H=(vPeriodTicks>>8)&0x0ff;

  
  //set channel 2mode
  //compare mode 100b, set on zero, clear on compare-up
  //enable compare interrupt
  T1CCTL2=0x64; //01100100; 

  
  //enable timer1
  //ϵͳʱΪ32MHzCLKCONCMD.TICKSPD[2:0]Ƶ.
  //T1CTL.DIVѡ00: Tick frequency/1
  //So it's 1/32us per tick
  //timer1 mode is Modulo
  CLKCONCMD &= 0x0c7; //bit5:3=000, 32MHz
  T1CTL=0x02; //00000010

  //Timer1ж
  IEN1=IEN1|0x02;

  return 0;

}

/**
 *  set pulse width
 */
int8 modPwmSetWidth(uint16 vWidth) {
  T1CC2L=vWidth&0xff;
  T1CC2H=(vWidth>>8)&0xff;

  return 0;
}

/**
 * Timer 1 жϷ
 */
#pragma vector=T1_VECTOR
__interrupt void TIMER1_IRQ(void){
  //ж
  EA=0;


  //жжԴ
  //жϱ־
  T1IF=0; //жϱ־

  if ((T1CCTL2&0x40)&&(T1STAT&0x10)) { //T1CCTL4.IM=1  CH4IF=1 (ch4Ƚж)
    //channel2 жϴ

    mPwmNumOfOverflowLeft --;
    if (mPwmNumOfOverflowLeft == 0) {
      mPwmNumOfOverflowLeft = mPwmNumOfOverflowCfg;
      //call user defined callback function
	  if (mPwmCallback) {
	    mPwmCallback();
	  }
    }

    //ch4ϴ
  }

  //Timer1жϱ־
  T1STAT= 0;

  //ж
  EA=1;

}

