/******************************************************************************
Filename:     modRfWithAck.c

Description:
H.Zheng 802.15.4 building block layer module library: RF part
Chip: TI CC2530

һּ򵥵modRf.hӿڵʵַʽ
ֱӷͷ飬֧ackֻ֧ACKطʹCC2530ӲAUTOACKܣ


modification:
2008.1.5 creation Zheng Hui
2011.5.30 modified to meet cc2530 usage, base on basicRF
2011.7.3  discard basicRFackط
2012.05.06 ¶Ĺӿ޸ġ㹦ܣMS-LRWPAN stackתƵmodRfStackС
           ļʵ·㹦ܣṩStackĵײ֧֡
2012.05.14 copy from modRfSimpleImpl.c֧ACK
2012.07.07 ޸modRfSendRawPktmodRfRxFrmDoneIsrdebugϢ

COPYRIGHT
Zheng Hui, Guangzhou, China. hzheng@gzhu.edu.cn, Guangzhou University

******************************************************************************/
//#include <string.h>
//#include <stdio.h>
#include <ioCC2530.h>
#include "modRf.h"
//#include "hal_dma.h"
//#include "basic_rf.h"
#include "hal_rf.h"
#include "hal_rf_security.h"
#include "hal_int.h"
#include "hal_mcu.h"
#include "lmodRfStack.h"
#include "modSys.h"
#include "modTimer.h"

#include "modUart.h"//for temply debug 0707

//Type Define

//char type in ROM memory
typedef  unsigned char const __code    ROMCHAR ;


typedef struct {
    uint16 myAddr;
    uint16 panId;
    uint8 channel;
    uint8 ackRequest;
    uint8 securityFlag;
    uint8* securityKey;
    uint8* securityNonce;
} modRfCfg_t;

typedef struct {
    uint8 txSeqNumber;
    volatile uint8 ackReceived;
    uint8 receiveOn;
    uint32 frameCounter;
} modRfTxState_t;

//constant
#define FLAG_FIELD                          0x09
#define NONCE_SIZE                          16

// Packet overhead ((frame control field, sequence number, destPANID, destAddr, srcPANID, srcAddr) + (footer))
#define MOD_RF_PACKET_HEADER_SIZE       (2 + 1 + 2 + 2 + 2 + 2 )
#define MOD_RF_FOOTER_SIZE                2
#define MOD_RF_PACKET_OVERHEAD_SIZE       (MOD_RF_PACKET_HEADER_SIZE + MOD_RF_FOOTER_SIZE)
#define MOD_RF_AUX_HDR_LENGTH             5
#define MOD_RF_ACK_PACKET_SIZE	          5
#define MOD_RF_SECURITY_M                 2
#define MOD_RF_LEN_MIC                    8

#define MOD_RF_MAX_PAYLOAD_SIZE	        (127 - MOD_RF_PACKET_OVERHEAD_SIZE - MOD_RF_AUX_HDR_LENGTH - MOD_RF_LEN_MIC)
#define MOD_RF_LEN_AUTH                   MOD_RF_PACKET_OVERHEAD_SIZE + MOD_RF_AUX_HDR_LENGTH - MOD_RF_FOOTER_SIZE

#define AUTO_CRC                   0x40
#define AUTO_ACK                   0x20
#define CC2530_TXPOWER_0_DBM       0xD5
#define IRQ_TXDONE                 0x02


#define MOD_RF_TX_RETRY           3

//parameter
static modRfCfg_t mRfConfig;
static modRfTxState_t mTxState=  { 0x00 };
static uint8 mRawRxMpdu[130];
static uint8 mTxDataMpdu[130];
//static uint8 mTxAckMpdu[10];
static uint8 mNonceTx[NONCE_SIZE];
static uint8 mNonceRx[NONCE_SIZE];

//added to disable address filtering. 2013.09.10
uint8 gRfRxNoFilterFlag =0;

//callback function
//for tx
static modRfTxCallbackFunction* mTxStartCallBackFunction=NULL;
static modRfTxCallbackFunction* mTxEndCallBackFunction=NULL;

//for receive
static modRfRxRawCallbackFunction* mRxRawCallBackFunction=NULL;
static modRfRxCallbackFunction* mRxCallBackFunction=NULL;
void modRfRxCallBack(void);
static void modRfRxFrmDoneIsr(void);
void modRfSecurityInit(void);
static uint8 modRfBuildMpdu(uint16 vDestAddr, uint8* vPayload, uint8 vPayloadLength);
static void modRfBuildMpduHeader(void);

//
int8 lmodRfIssueTransmit();

//for test
uint16 mAckDelayUs = 0;

/**
 * Ƶʼýӿ
 * 
 *     vNodeId: ڵַȡֵΧ0-0xFFFE0x0FFFFΪ㲥ַã
 *     vPanId: IDͬPanIdĽڵſ໥ͨţȡֵΧ0-0xFFFE0x0FFFFݲʹã
 *     vChannel: ƵŵIDȡֵΧ淶أ
 *     vAckRequestFlag: ·Ϣȷϱ־ʾ͵ķǷҪԷȷϢ0ΪҪ0ΪҪ
 *     vSecurityFlag: ·ܱ־ʾ͵ķǷҪܣ0ΪҪ0ΪҪҪʱvKeyΪ0
 *     vKey: ·Կָ룬ָԿԿʵֻأҪʱΪ0
 *
 * ע⣺ʵδ֧Ackͼܣ
 */
int8 modRfInit(uint16 vNodeId, uint16 vPanId, uint8 vChannel, uint8 vAckRequestFlag,
               uint8 vSecurityFlag, uint8 *vKey){

  mRfConfig.myAddr = vNodeId;
  mRfConfig.panId = vPanId;
  mRfConfig.channel = vChannel;
  mRfConfig.ackRequest = vAckRequestFlag;
  mRfConfig.securityFlag = vSecurityFlag;
  if (vSecurityFlag > 0) {
    mRfConfig.securityKey = vKey;
  }

  mTxState.receiveOn = 1;
  mTxState.frameCounter = 0;
  //
  modRfBuildMpduHeader(); //init the header part of mTxDataMpdu


  //halRfInit()
  // Enable auto ack and auto crc
  //FRMCTRL0 |= (AUTO_ACK | AUTO_CRC);
  //FRMCTRL0 |= AUTO_CRC;
  TXPOWER = CC2530_TXPOWER_0_DBM;      // TX output power ~0 dBm
  TXFILTCFG = 0x09;
  AGCCTRL1 = 0x15;
  FSCAL1 = 0x00;
  //FRMFILT0 = FRMFILT0&0xFE; //clear bit0, disable frame filtering
  //
  if (vAckRequestFlag>0) { //enable ACK
    FRMCTRL0 |= (AUTO_ACK | AUTO_CRC);
  }else {
    FRMCTRL0 |= AUTO_CRC;
    FRMFILT0 = FRMFILT0&0xFE; //clear bit0, disable frame filtering
  }

  halIntOff();


  // Set channel
  halRfSetChannel(vChannel);

  // Write the short address and the PAN ID to the chip
  halRfSetShortAddr(vNodeId);
  halRfSetPanId(vPanId);

  // if security is enabled, write key and nonce
  if (vSecurityFlag > 0) {
    modRfSecurityInit();
  }

  // Set up receive interrupt service handler
  halRfRxInterruptConfig(modRfRxFrmDoneIsr);

  // Enable RX interrupt
  halRfEnableRxInterrupt();

  //
  modRfStartRx();

  //
  halIntOn();

  return 0;
}


/**
 * ͷ
 */
static uint8 modRfBuildMpdu(uint16 vDestAddr, uint8* vPayload, uint8 vPayloadLength) {
  uint8 i,iPktSize;

  for(i=0;i<vPayloadLength;i++)  {
      mTxDataMpdu[1+MOD_RF_PACKET_HEADER_SIZE+i] = vPayload[i];
  }
  //
  iPktSize = 1 + MOD_RF_PACKET_HEADER_SIZE + vPayloadLength + MOD_RF_FOOTER_SIZE;
  mTxDataMpdu[0]=iPktSize-1; //ȥsizeռõһֽ
  mTxDataMpdu[3]= mTxState.txSeqNumber;
  mTxDataMpdu[6]=vDestAddr&0xFF;  //dest addr low byte
  mTxDataMpdu[7]=(vDestAddr>>8)&0xFF; //destination address high byte

  return iPktSize; // total mpdu length
}

/**
 * ͷheader
 */
static void modRfBuildMpduHeader() {
  //mTxDataMpdu[0]=pktSize;
  //mTxDataMpdu[1]=0x88; //frame control
  //mTxDataMpdu[2]=0x01;
  if (mRfConfig.ackRequest>0) {
    mTxDataMpdu[1]=0x21; //frame control
    mTxDataMpdu[2]=0x88;
  }else {
    mTxDataMpdu[1]=0x01; //frame control
    mTxDataMpdu[2]=0x88;
  }

  //mTxDataMpdu[3]= mSeqNumber;
  mTxDataMpdu[4]=mRfConfig.panId&0xFF;  //low byte
  mTxDataMpdu[5]=(mRfConfig.panId>>8)&0xFF; //dest PAN ID high byte
  //mTxDataMpdu[6]=(destAddr>>8)&0xFF;
  //mTxDataMpdu[7]=destAddr&0xFF;
  mTxDataMpdu[8]=mRfConfig.panId&0xFF;
  mTxDataMpdu[9]=(mRfConfig.panId>>8)&0xFF;
  mTxDataMpdu[10]=mRfConfig.myAddr&0xFF;
  mTxDataMpdu[11]=(mRfConfig.myAddr>>8)&0xFF;

}

/**
 * security init
 */
void modRfSecurityInit(){
    uint8 i;

    // Initialise nonce bytes to 0
    for(i=0;i<NONCE_SIZE;i++)
    {
        mNonceRx[i] = 0;
        mNonceTx[i] = 0;
    }

    // Set nonce flag field (Byte 0)
    mNonceRx[0] = FLAG_FIELD;
    mNonceTx[0] = FLAG_FIELD;

    // Set byte 7 and 8 of nonce to myAddr
    mNonceTx[8] = (uint8)mRfConfig.myAddr;
    mNonceTx[7] = (uint8)(mRfConfig.myAddr>>8);

    // Set Security mode field of nonces (Byte 13)
    mNonceRx[13] = SECURITY_CONTROL;
    mNonceTx[13] = SECURITY_CONTROL;

    halRfSecurityInit(mRfConfig.securityKey, mNonceRx, mNonceTx);
}



/**
 * ͷӿ
 * 
 *     vDestinationId: Զ˵ַȡֵΧ0-0xFFFF0x0FFFFΪ㲥
 *     vSize: ͵û鳤ȣֽĿȡֵΧԼ·ʵֻйأ
 *     vPacket: ûָ룬ָ͵û飻
 */
int8 modRfSendPkt(uint16 vDestinationId, uint8 vSize, uint8* vPacket) {
  uint8 iMpduLength;
  int8 ret=0;
  uint8 iTxRetry;
  uint16 i;


  // Turn on receiver if its not on
  if(!mTxState.receiveOn) {
      halRfReceiveOn();
  }

  // Wait until the transceiver is idle
  halRfWaitTransceiverReady();

  // Turn off RX frame done interrupt to avoid interference
  halRfDisableRxInterrupt();

  //build the packet to be sent
  iMpduLength = modRfBuildMpdu(vDestinationId, vPacket, vSize);

  //write packet data to RFD buffer
  /*
  if (mRfConfig.securityFlag>0) {
    halRfWriteTxBufSecure(mTxDataMpdu, iMpduLength, vSize, MOD_RF_LEN_AUTH, MOD_RF_SECURITY_M);
    mTxState.frameCounter++;     // Increment frame counter field
    halRfIncNonceTx();          // Increment nonce value
  }else {
    halRfWriteTxBuf((uint8*)mTxDataMpdu, iMpduLength);
  }
  */
  halRfWriteTxBuf((uint8*)mTxDataMpdu, iMpduLength);


  //send packet

  // Send frame with CCA. return FAILED if not successful
  /*
  if(halRfTransmit() != SUCCESS) {
    ret = FAILED;
  }else {
    ret = SUCCESS;
  }
  */


  if (mRfConfig.ackRequest >0) {
    mTxState.ackReceived = 0;
    //enable retry
    iTxRetry = MOD_RF_TX_RETRY;
    while (iTxRetry >0) {
      ret = lmodRfIssueTransmit();
      if (ret == 0) {
        //tx success, wait for ACK

        // Turn on RX frame done interrupt for ACK reception
        halRfEnableRxInterrupt();

        ret = -1;

        for (i=0;i<3000;i++) {//wait 3ms before re-tx  (normal is about 300us)
          //delay 1us
          {
            asm("nop");
            asm("nop");
            asm("nop");
            asm("nop");
            asm("nop");
            asm("nop");
            asm("nop");
            asm("nop");
            asm("nop");
            asm("nop");
          }

          if (mTxState.ackReceived>0) {
            ret = 0;
            break; //break for loop
          }
        }
        if (ret == 0) { //ACK received
          mAckDelayUs = i;
          break; //break while loop
        }
        //else wait and retry
        halMcuWaitMs(5);
      }else {
        halMcuWaitUs(100);
      }
      iTxRetry --;
    }
  }else { //no ack
    ret = lmodRfIssueTransmit(); //direct send
  }



  // Turn off the receiver if it should not continue to be enabled
  if (!mTxState.receiveOn) {
      halRfReceiveOff();
  }

  if(ret == 0) {
      mTxState.txSeqNumber++;
  }

  // Turn on RX frame done interrupt for ACK reception
  halRfEnableRxInterrupt();

  return ret;

}

/**
 * ִз
 */
int8 lmodRfIssueTransmit(){
  uint8 iDelayCount;
  int8 ret=0;
  //wait for CCA for a specified period
  iDelayCount =100;
  while (!(FSMSTAT1&0x10)) { //FSMSTAT1.4=0
    halMcuWaitUs(100);
    iDelayCount --;
    if (iDelayCount == 0) {
      ret = -1;
      break;
    }
  }

  if (FSMSTAT1&0x10) { //CCA set
    ret = 0;
    //transmit
    ISTXON(); // Sending

    // Waiting for transmission to finish
    iDelayCount =100;
    while(!(RFIRQF1 & IRQ_TXDONE) ) {
      halMcuWaitUs(100);
      iDelayCount --;
      if (iDelayCount == 0) {
        ret = -1;
        break;
      }
    }

    RFIRQF1 = ~IRQ_TXDONE;
  }else {
    return -1;
  }
  return ret;
}



/**
 * ýշصӿ
 * 
 *     vFunction: ûƵķصָ룻
 */
int8 modRfSetRxCallback(modRfRxCallbackFunction* vFunction ){
  mRxCallBackFunction = vFunction;
  return 0;
}

/**
 * ģʽӿ
 */
int8 modRfStartRx(){
  mTxState.receiveOn = TRUE;
  halRfReceiveOn();
  return 0;
}

/**
 * ͹ýӿ
 * 
 *     vPowerLevel: 书ʵȼ0Ϊ0dBmӦ0dBmӦ0dBmӦϵӲأ
 * CC2530ʵ֣(default =0xF5)
 * vPowerLevel=3: power=4.5dBm; registerValue=0xF5;
 * vPowerLevel=2: power=2.5dBm; registerValue=0xE5;
 * vPowerLevel=1: power=1dBm; registerValue=0xD5;
 * vPowerLevel=0: power=-0.5dBm; registerValue=0xC5;
 * vPowerLevel=-1: power=-3dBm; registerValue=0xA5;
 * vPowerLevel=-2: power=-6dBm; registerValue=0x85;
 * vPowerLevel=-3: power=-12dBm; registerValue=0x55;
 * vPowerLevel=-4: power=-18dBm; registerValue=0x25;
 * vPowerLevel=-5: power=-22dBm; registerValue=0x05;

 */
int8 modRfSetTxPower(int8 vPowerLevel){
  //map vPowerLevel to register value
    switch(vPowerLevel)  {
    case (3):
      TXPOWER=0xF5;
      break;
    case (2):
      TXPOWER=0xE5;
      break;
    case (1):
      TXPOWER=0xD5;
      break;
    case (0):
      TXPOWER=0xC5;
      break;
    case (-1):
      TXPOWER=0xA5;
      break;
    case (-2):
      TXPOWER=0x85;
      break;
    case (-3):
      TXPOWER=0x55;
      break;
    case (-4):
      TXPOWER=0x25;
      break;
    case (-5):
      TXPOWER=0x05;
      break;
    default:
      return MOD_RET_NOT_SUPPORT;
    }

  return 0;
}






/**
 * ͷӿ: LR_WPAN stack
 * 2012.07.07 add stack debug support. add send time
 */
signed char modRfSendRawPkt(unsigned char vSize, unsigned char* vPacket) {
  int8 ret=0;
  uint8 iDelayCount;
  uint8 i;
  uint16 iTime;


  // Turn on receiver if its not on
  if(!mTxState.receiveOn) {
      halRfReceiveOn();
  }

  // Wait until the transceiver is idle
  halRfWaitTransceiverReady();

  // Turn off RX frame done interrupt to avoid interference
  halRfDisableRxInterrupt();

  //send packet
  ISFLUSHTX();          // Making sure that the TX FIFO is empty.
  RFIRQF1 = ~IRQ_TXDONE;   // Clear TX done interrupt

  //add stack debug support. add send time
  /* not work
  if (gModStackDebugFlag>0) {
    iTime = gNetSyncTimerCounter&0x0FFFF;
    RFD = vSize+2+2;
    // Insert data
    for(i=0;i<vSize;i++){
      RFD = vPacket[i];
    }
    // Insert time
    RFD = (uint8)((iTime>>8)&0x0ff);
    RFD = (uint8)(iTime&0x0ff);

  }else {
    RFD = vSize+2;
    // Insert data
    for(i=0;i<vSize;i++){
      RFD = vPacket[i];
    }
  }
  */
  if (gModStackDebugFlag>0) {
    iTime = gNetSyncTimerCounter&0x0FFFF;
    if ((vPacket[vSize-2]==0x7e)&&(vPacket[vSize-1]==0x5f)) {
      vPacket[vSize-4]=(uint8)((iTime>>8)&0x0ff);
      vPacket[vSize-3]=(uint8)(iTime&0x0ff);
    }
  }
  RFD = vSize+2;
  // Insert data
  for(i=0;i<vSize;i++){
    RFD = vPacket[i];
  }


  //wait for CCA for a specified period
  iDelayCount =100;
  while (!(FSMSTAT1&0x10)) { //FSMSTAT1.4=0
    halMcuWaitUs(100);
    iDelayCount --;
    if (iDelayCount == 0) {
      ret = -1;
      break;
    }
  }
  //if (FSMSTAT1&0x10) { //CCA set
  if (ret == 0) { //CCA set
    ISTXON(); // Sending
    //callback
    if(mTxStartCallBackFunction) {
       mTxStartCallBackFunction();
    }


    // Waiting for transmission to finish
    iDelayCount =100;
    while(!(RFIRQF1 & IRQ_TXDONE) ) {
      halMcuWaitUs(100);
      iDelayCount --;
      if (iDelayCount == 0) {
        ret = -1;
        break;
      }
    }
    if (ret == 0) {
      //callback
      if(mTxEndCallBackFunction) {
         mTxEndCallBackFunction();
      }

    }

    RFIRQF1 = ~IRQ_TXDONE;
  }

  // Turn off the receiver if it should not continue to be enabled
  if (!mTxState.receiveOn) {
      halRfReceiveOff();
  }

  if(ret == 0) {
      mTxState.txSeqNumber++;
  }

  // Turn on RX frame done interrupt for ACK reception
  halRfEnableRxInterrupt();

  return ret;

}

/**
 * Ƶŵýӿ
 */
void modSetRfChannel(uint8 vChannel) {
  mRfConfig.channel = vChannel;
  halRfSetChannel(vChannel);
}
/**
 * ڵַýӿ
 */
void modSetRfNodeId(unsigned short vNodeId) {
  mRfConfig.myAddr = vNodeId;
  halRfSetShortAddr(vNodeId);  //Write short address to chip
  modRfBuildMpduHeader();
}

unsigned short modGetRfNodeId() {
  return mRfConfig.myAddr;
}
/**
 * PANַýӿ
 */
void modSetRfPanId(unsigned short vPanId) {
  mRfConfig.panId = vPanId;
  halRfSetPanId(vPanId); //Write PAN Id to chip
  modRfBuildMpduHeader();
}

unsigned short modGetRfPanId() {
  return mRfConfig.panId;
}


/**
 * ȡFlashеIEEEַӿ (עֽ˳ֽǰûеԭǱȽʱ췢ֲͬ)
 */
#define IEEE_ADDRESS_ARRAY 0xFFE8

void modGetProcessorIEEEAddress(unsigned char *vBuf) {
  unsigned char bank;
  uint8 iInterrupt;
  uint8 *ptr;
  //save and disable interrupt
  iInterrupt = EA;
  EA = 0;
  //
  bank = MEMCTR;
  //switch to bank 7
   MEMCTR = (MEMCTR & 0xF8) | 0x07;
   //
   ptr = (uint8 *)0xFFE8;
   vBuf[0]= *ptr++;
   vBuf[1]= *ptr++;
   vBuf[2]= *ptr++;
   vBuf[3]= *ptr++;
   vBuf[4]= *ptr++;
   vBuf[5]= *ptr++;
   vBuf[6]= *ptr++;
   vBuf[7]= *ptr++;

  //resore old bank settings
  MEMCTR = bank;
  //restore interrupt config
  EA = iInterrupt;
}

/**
 * üĴеIEEEַ
 */
void modSetRfIEEEAddress(unsigned char *vBuf) {
  EXT_ADDR0 = vBuf[0];
  EXT_ADDR1 = vBuf[1];
  EXT_ADDR2 = vBuf[2];
  EXT_ADDR3 = vBuf[3];
  EXT_ADDR4 = vBuf[4];
  EXT_ADDR5 = vBuf[5];
  EXT_ADDR6 = vBuf[6];
  EXT_ADDR7 = vBuf[7];

}


/**
 * ÷ͷصӿ
 * appģڸúʵֶԷĴ
 */

void modRfSetTxStartCallback(modRfTxCallbackFunction* vFunction) {
  mTxStartCallBackFunction = vFunction;
}
void modRfSetTxEndCallback(modRfTxCallbackFunction* vFunction) {
  mTxEndCallBackFunction = vFunction;
}



/**
 * ýշصӿ
 * appģڸúʵֶԷĴ
 */

void modRfSetRxRawCallback(modRfRxRawCallbackFunction* vFunction) {
  mRxRawCallBackFunction = vFunction;
}




/**
 * жϷmodRFƬϣ
 * halRFеĽжϷ
 * 2012.07.07 add stack debug support. add received time for RxRawCallback. (NOT WORK)
 */
static void modRfRxFrmDoneIsr(void) {
  uint8 iPktLength;
  //uint16 iSrcAddr, iDestAddr, iDestPanId;
  uint16 iSrcAddr, iDestAddr;
  uint8 iRssi8;
  int16 iRssi16;
  uint8 iCrc;
  uint16 iTime;

  // Clear interrupt and disable new RX frame done interrupt
  halRfDisableRxInterrupt();

  // Enable all other interrupt sources (enables interrupt nesting)
  //halIntOn();

  //Read payload length
  halRfReadRxBuf(mRawRxMpdu,1);
  iPktLength = mRawRxMpdu[0]&0x7F;
  //read payload
  halRfReadRxBuf(mRawRxMpdu+1, iPktLength);

  //ʹⲿstackʱmRxRawCallBackFunction!=0
  if(mRxRawCallBackFunction) {

    //2012.07.07 add stack debug support. add received time
    //not work. RFD can not join
    if (gModStackDebugFlag>0) {

      //for 0707 temply debug only
      //  modUartSendPkt(0,iPktLength,mRawRxMpdu);
      //

    /*
      iTime = gNetSyncTimerCounter&0x0FFFF;
      //modify packet
      mRawRxMpdu[0]=iPktLength+2;
      mRawRxMpdu[iPktLength+1]=mRawRxMpdu[iPktLength-1];
      mRawRxMpdu[iPktLength]=mRawRxMpdu[iPktLength-2];
      mRawRxMpdu[iPktLength-2]= (uint8)((iTime>>8)&0x0ff);
      mRawRxMpdu[iPktLength-1]= (uint8)(iTime&0x0ff);
      //deliver packet
      mRxRawCallBackFunction(iPktLength+2, mRawRxMpdu);
    */
      iTime = gNetSyncTimerCounter&0x0FFFF;
      if ((mRawRxMpdu[iPktLength-3]==0x7e)&&(mRawRxMpdu[iPktLength-2]==0x5f)) {
        mRawRxMpdu[iPktLength-3]= (uint8)((iTime>>8)&0x0ff);
        mRawRxMpdu[iPktLength-2]= (uint8)(iTime&0x0ff);
      }
      //deliver packet
      mRxRawCallBackFunction(iPktLength, mRawRxMpdu);
    }else {
      mRxRawCallBackFunction(iPktLength, mRawRxMpdu);
    }
  }else {
    //check whether ACK packet
    if (iPktLength == 5) { //ACK packet
      //verify CRC and tx pkt sequence
      iCrc=mRawRxMpdu[iPktLength]&0x80;
      if ((iCrc>0) && (mRawRxMpdu[3] == mTxState.txSeqNumber)) {
        mTxState.ackReceived = 1;
      }
      //
    }else { //data packet
      if (iPktLength>10) {
        //read header fields
        //iDestPanId = (mRawRxMpdu[4]&0xff)*256+(mRawRxMpdu[5]&0xff);
        iDestAddr = (mRawRxMpdu[7]&0xff)*256+(mRawRxMpdu[6]&0xff);
        iSrcAddr = (mRawRxMpdu[11]&0xff)*256+(mRawRxMpdu[10]&0xff);
        //
        iRssi8 = mRawRxMpdu[iPktLength-1];
        if(iRssi8 < 128){
          iRssi16 = iRssi8 - halRfGetRssiOffset();
        } else{
          iRssi16 = (iRssi8 - 256) - halRfGetRssiOffset();
        }
        //
        iCrc=mRawRxMpdu[iPktLength]&0x80;

        //filter
        if (gRfRxNoFilterFlag) {
          if(mRxCallBackFunction) {
            //mRxCallBackFunction(iSrcAddr, iDestAddr, iPktLength-1, mRawRxMpdu, iRssi16); 
            //mRxCallBackFunction(iSrcAddr, iDestAddr, iPktLength-11-2, mRawRxMpdu+12, iRssi16); //ûָصûݳȣpktLengthȥheader11ֽڣָʾֽڱԼRSSI2ֽڡ
            mRxCallBackFunction(iSrcAddr, iDestAddr, iPktLength-11, mRawRxMpdu+12, iRssi16); //ûָصûݳȣpktLengthȥheader11ֽڣָʾֽڱ
          }
        }else {
          if (iCrc>0) {
            if((iDestAddr == mRfConfig.myAddr) || (iDestAddr == MOD_BROADCAST_ADDRESS)) { //ĿĵǱڵַ㲥ַ
              if(mRxCallBackFunction) {
                mRxCallBackFunction(iSrcAddr, iDestAddr, iPktLength-11-2, mRawRxMpdu+12, iRssi16); //ûָصûݳȣpktLengthȥheader11ֽڣָʾֽڱԼRSSI2ֽڡ
              }
            }
          }
        }
        
      }
    }
  }

  // Enable RX frame done interrupt again
  //halIntOff();
  halRfEnableRxInterrupt();
}

